package Time::OO;
use strict;
use POSIX 'strftime';
use Data::Dumper;
use Time::Local;
use Time::OO::ModuleCalc;
#use Time::OO::Week;
use Time::OO::Second;
use Time::OO::Minute;
use Time::OO::Hour;
use Time::OO::Day;
use Time::OO::Month;
use Time::OO::Year;
use Time::OO::Decade;
use Time::OO::Century;
use Time::OO::Millenium;


sub new {
    my $class = shift();

    my $self = {
	's_dirty' => 1,
    };
    bless($self, $class);

    $self->{'second'}      = Time::OO::Second->new($self);
    $self->{'minute'}      = Time::OO::Minute->new($self);
    $self->{'hour'}        = Time::OO::Hour->new($self);
    $self->{'day'}        = Time::OO::Day->new($self);
#    $self->{'day_of_week'} = Time::OO::ModuleCalc->new($self);
#    $self->{'week'}      = Time::OO::Week->new($self);
    $self->{'month'}     = Time::OO::Month->new($self);
    $self->{'year'}      = Time::OO::Year->new($self);
    $self->{'decade'}    = Time::OO::Decade->new($self);
    $self->{'century'}   = Time::OO::Century->new($self);
    $self->{'millenium'} = Time::OO::Millenium->new($self);
    $self->{'symbolic'}  = Time::OO::Symbolic->new($self);

    return($self);
}


sub second    {  return($_[0]->{'second'})    }
sub minute    {  return($_[0]->{'minute'})    }
sub hour      {  return($_[0]->{'hour'})      }
sub day       {  return($_[0]->{'day'})       }
sub month     {  return($_[0]->{'month'})     }
sub year      {  return($_[0]->{'year'})      }
sub decade    {  return($_[0]->{'decade'})    }
sub century   {  return($_[0]->{'century'})   }
sub millenium {  return($_[0]->{'millenium'}) }
sub symbolic  {  return($_[0]->{'symbolic'})  }


sub remaining {
    my $self = shift();
    return(
	   $self->second->remaining(),
	   $self->minute->remaining(),
	   $self->hour->remaining(),
	   $self->day->remaining(),
	   $self->month->remaining(),
	   $self->year->remaining()
	   );
}


sub import {
    my $self = shift();
    my $value = shift();

    if (defined($value)) {
	$self->{'value'} = $value;
	my  $year = 0;
	($self->second->{'value'},
	 $self->minute->{'value'},
	 $self->hour->{'value'},
	 $self->day->{'value'},
	 $self->month->{'value'},
	 $year) = (localtime($value))[0,1,2,3,4,5];
	 $self->year->{'value'} = $year +1900;
    } else {
#	return($self->{'value'});
	return($self);
    }
}


sub as_string {
    my $self = shift();
    my $format = shift();

#    print STDERR Dumper($self->day());
    my $string = strftime($format,
			  ($self->second->value(),
			  $self->minute->value(),
			  $self->hour->value(),
			  $self->day->value(),
			  $self->month->value(),
			  $self->year->value()-1900)
			  );
    return($string);
}


sub seconds {
    my $self = shift();

    my $s = timelocal(
		      $self->second->value(),
		      $self->minute->value(),
		      $self->hour->value(),
		      $self->day->value(),
		      $self->month->value(),
		      $self->year->value()-1900
		      );
    return($s);
}


1;
