package Time::OO::Year;
use strict;
use vars qw($VERSION @ISA);
use Time::OO::ModuleCalc;
@ISA = ("Time::OO::ModuleCalc");
$VERSION = sprintf("%d.%d.%d", q$Revision: 1.0.0 $ =~ /(\d+)\.(\d+)\.(\d+)/);


sub init {
    my $self = shift();

    $self->{'limit'} = 2^31;
    $self->{'upper'} = "year";
    $self->{'german_week'} = 1;
    $self->{'days'} = {
	'normal' => 365,
	'leap'   => 366,
    };
}


sub is_leapyear {
    my $self = shift();
    my $year = shift();

    $year = $self->{'value'} unless ($year);
    unless (exists($self->{'leap_cache'}{$year})) {
	if      (($self->{'value'} % 400) == 0) {
	    $self->{'leap_cache'}{$year} = 1;
	} elsif (($self->{'value'} % 100) == 0) {
	    $self->{'leap_cache'}{$year} = 0;
	} elsif (($self->{'value'} % 4)   == 0) {
	    $self->{'leap_cache'}{$year} = 1;
	} else {
	    $self->{'leap_cache'}{$year} = 0;
	}
    }
    return($self->{'leap_cache'}{$year});
}


sub seconds {
    my $self = shift();
    my $year = shift();

    $year = $self->{'value'} unless ($year);
    return($self->days()*86400);
}


sub minutes {
    my $self = shift();
    my $year = shift();

    $year = $self->{'value'} unless ($year);
    return($self->days($year)*1440);
}


sub hours {
    my $self = shift();
    my $year = shift();

    $year = $self->{'value'} unless ($year);
    return($self->days($year)*24);
}


sub weeks {
    my $self = shift();
    my $year = shift();

    $year = $self->{'value'} unless ($year);
    return(52) if ($self->weekday_of_forth($year) > 4);
    return(53);
}


sub weekday_of_forth {
    my $self = shift();
    my $year = shift();

    $year = $self->{'value'} unless ($year);
    unless (exists($self->{'wof'}{$year})) {
        my $time = timelocal(0, 0, 0, 4, 0, $year);
        my $wof = (localtime($time))[6];
        $wof += 7 unless ($wof && $self->{'german_week'});
	$self->{'wof'}{$year} = $wof;
    }
    return($self->{'wof'}{$year});
}



sub days {
    my $self = shift();
    my $year = shift();

    $year = $self->{'value'} unless ($year);
    my $days = 0;
    if ($self->is_leapyear($year)) {
	$days = $self->{'days'}->{'leap'};
    } else {
	$days = $self->{'days'}->{'normal'};
    }
    return($days);
}


sub months () {
    return(12);
}


sub remaining () {
    my $self = shift();
    warn( ref($self).": do you believe in the end of times?" );
}


sub normalize {
    my $self = shift();
    my $much = shift();
    return($much);
}


1;
